import { ResolverBatch } from '@stringifyrjs/core/src/sfyr/ResolverBatch';
import { TAnySfyr } from '@stringifyrjs/core/src/sfyr/StringifyrInstance';
import * as React from 'react';
import { PropsWithChildren } from 'react';
import { useStringifyr } from './useStringifyr';

export const FetchBoundaryContext = React.createContext({
  sfyr: null as TAnySfyr,
});

type TProps = {
  //
};

export const StringifyrFetchBoundary = ({
  children,
}: PropsWithChildren<TProps>) => {
  const parent = useStringifyr();

  const [sfyr, setSfyr] = React.useState<TAnySfyr>(() => {
    const result = parent.clone({
      fetched: false,
      resolverBatch: new ResolverBatch(),
    });

    return result;
  });

  React.useEffect(() => {
    if (sfyr.fetched) {
      return;
    }

    (async () => {
      await new Promise((resolve) => setTimeout(resolve, 1000));
      const next = await sfyr.fetch();
      setSfyr(next);
    })();
  }, [sfyr, setSfyr]);

  const value = React.useMemo(() => ({ sfyr }), [sfyr]);
  return (
    <FetchBoundaryContext.Provider value={value}>
      {children}
    </FetchBoundaryContext.Provider>
  );
};
